<?php
/**
 * File containing the VersatileMatcher class.
 *
 * @copyright Copyright (C) 1999-2014 eZ Systems AS. All rights reserved.
 * @license http://ez.no/licenses/gnu_gpl GNU General Public License v2.0
 * @version 
 */

namespace eZ\Publish\Core\MVC\Symfony\SiteAccess;

/**
 * Interface for SiteAccess matchers.
 *
 * VersatileMatcher makes it possible to do a reverse match (e.g. "Is this matcher knows provided SiteAccess name?").
 * Versatile matchers enable cross-siteAccess linking.
 */
interface VersatileMatcher extends Matcher
{
    /**
     * Returns matcher object corresponding to $siteAccessName or null if non applicable.
     *
     * @note VersatileMatcher objects always receive a request with cleaned up pathinfo (i.e. no SiteAccess part inside).
     *
     * @param string $siteAccessName
     *
     * @return \eZ\Publish\Core\MVC\Symfony\SiteAccess\VersatileMatcher|null Typically the current matcher, with updated request.
     */
    public function reverseMatch( $siteAccessName );

    /**
     * Returns the SimplifiedRequest object corresponding to the reverse match.
     * This request object can then be used to build a link to the "reverse matched" SiteAccess.
     *
     * @see reverseMatch()
     *
     * @return \eZ\Publish\Core\MVC\Symfony\Routing\SimplifiedRequest
     */
    public function getRequest();
}
