<?php
/**
 * File containing the DateMapper class
 *
 * @copyright Copyright (C) 1999-2014 eZ Systems AS. All rights reserved.
 * @license http://ez.no/licenses/gnu_gpl GNU General Public License v2.0
 * @version 
 */

namespace eZ\Publish\Core\Persistence\Solr\Content\Search\FieldValueMapper;

use eZ\Publish\Core\Persistence\Solr\Content\Search\FieldValueMapper;
use eZ\Publish\SPI\Persistence\Content\Search\Field;
use eZ\Publish\SPI\Persistence\Content\Search\FieldType;
use DateTime;
use InvalidArgumentException;
use Exception;

/**
 * Maps raw document field values to something Solr can index.
 */
class DateMapper extends FieldValueMapper
{
    /**
     * Check if field can be mapped
     *
     * @param Field $field
     *
     * @return mixed
     */
    public function canMap( Field $field )
    {
        return $field->type instanceof FieldType\DateField;
    }

    /**
     * Map field value to a proper Solr representation
     *
     * @param Field $field
     *
     * @return mixed
     */
    public function map( Field $field )
    {
        if ( is_numeric( $field->value ) )
        {
            $date = new DateTime( "@{$field->value}" );
        }
        else
        {
            try
            {
                $date = new DateTime( $field->value );
            }
            catch ( Exception $e )
            {
                throw new InvalidArgumentException( "Invalid date provided: " . $field->value );
            }
        }

        return $date->format( "Y-m-d\\TH:i:s\\Z" );
    }
}
