<?php
/**
 * File containing the Root ValueObjectVisitor class
 *
 * @copyright Copyright (C) 1999-2014 eZ Systems AS. All rights reserved.
 * @license http://ez.no/licenses/gnu_gpl GNU General Public License v2.0
 * @version 
 */

namespace eZ\Publish\Core\REST\Server\Output\ValueObjectVisitor;

use eZ\Publish\Core\REST\Common\Output\ValueObjectVisitor;
use eZ\Publish\Core\REST\Common\Output\Generator;
use eZ\Publish\Core\REST\Common\Output\Visitor;

/**
 * Root value object visitor
 */
class Root extends ValueObjectVisitor
{
    /**
     * Visit struct returned by controllers
     *
     * @param \eZ\Publish\Core\REST\Common\Output\Visitor $visitor
     * @param \eZ\Publish\Core\REST\Common\Output\Generator $generator
     * @param \eZ\Publish\Core\REST\Common\Values\Root $data
     */
    public function visit( Visitor $visitor, Generator $generator, $data )
    {
        $generator->startObjectElement( 'Root' );
        $visitor->setHeader( 'Content-Type', $generator->getMediaType( 'Root' ) );

        // Uses hashElement instead of objectElement as a workaround to be able to have
        // an empty media-type, since there is no media type for list of content yet
        $generator->startHashElement( 'content' );
        $generator->startAttribute( 'media-type', '' );
        $generator->endAttribute( 'media-type' );
        $generator->startAttribute(
            'href',
            $this->router->generate( 'ezpublish_rest_createContent' )
        );
        $generator->endAttribute( 'href' );
        $generator->endHashElement( 'content' );

        // Uses hashElement instead of objectElement as a workaround to be able to have
        // an empty media-type, since there is no media type for list of content yet
        $generator->startHashElement( 'contentByRemoteId' );
        $generator->startAttribute( 'media-type', '' );
        $generator->endAttribute( 'media-type' );
        $generator->startAttribute(
            'href',
            $this->templateRouter->generate(
                'ezpublish_rest_redirectContent',
                array( 'remoteId' => '{remoteId}' )
            )
        );
        $generator->endAttribute( 'href' );
        $generator->endHashElement( 'contentByRemoteId' );

        // Content types list
        $generator->startObjectElement( 'contentTypes', 'ContentTypeInfoList' );
        $generator->startAttribute( 'href', $this->router->generate( 'ezpublish_rest_listContentTypes' ) );
        $generator->endAttribute( 'href' );
        $generator->endObjectElement( 'contentTypes' );

        // Content type by identifier
        // See comment above regarding usage of hash element
        $generator->startHashElement( 'contentTypeByIdentifier' );
        $generator->startAttribute( 'media-type', '' );
        $generator->endAttribute( 'media-type' );
        $generator->startAttribute(
            'href',
            $this->templateRouter->generate(
                'ezpublish_rest_listContentTypes',
                array( 'identifier' => '{identifier}' )
            )
        );
        $generator->endAttribute( 'href' );
        $generator->endHashElement( 'contentTypeByIdentifier' );

        // Creating a ContentTypeGroup
        $generator->startObjectElement( 'contentTypeGroups', 'ContentTypeGroupList' );
        $generator->startAttribute( 'href', $this->router->generate( 'ezpublish_rest_createContentTypeGroup' ) );
        $generator->endAttribute( 'href' );
        $generator->endObjectElement( 'contentTypeGroups' );

        // Content type group by identifier
        // See comment above regarding usage of hash element
        $generator->startHashElement( 'contentTypeGroupByIdentifier' );
        $generator->startAttribute( 'media-type', '' );
        $generator->endAttribute( 'media-type' );
        $generator->startAttribute(
            'href',
            $this->templateRouter->generate(
                'ezpublish_rest_loadContentTypeGroupList',
                array( 'identifier' => '{identifier}' )
            )
        );
        $generator->endAttribute( 'href' );
        $generator->endHashElement( 'contentTypeGroupByIdentifier' );

        // Users
        $generator->startObjectElement( 'users', 'UserRefList' );
        $generator->startAttribute( 'href', $this->router->generate( 'ezpublish_rest_loadUsers' ) );
        $generator->endAttribute( 'href' );
        $generator->endObjectElement( 'users' );

        // Roles
        $generator->startObjectElement( 'roles', 'RoleList' );
        $generator->startAttribute( 'href', $this->router->generate( 'ezpublish_rest_listRoles' ) );
        $generator->endAttribute( 'href' );
        $generator->endObjectElement( 'roles' );

        // Content root location
        $generator->startObjectElement( 'rootLocation', 'Location' );
        $generator->startAttribute( 'href', $this->router->generate( 'ezpublish_rest_loadLocation', array( 'locationPath' => '1/2' ) ) );
        $generator->endAttribute( 'href' );
        $generator->endObjectElement( 'rootLocation' );

        // Users root location
        $generator->startObjectElement( 'rootUserGroup', 'UserGroup' );
        $generator->startAttribute( 'href', $this->router->generate( 'ezpublish_rest_loadUserGroup', array( 'groupPath' => '1/5' ) ) );
        $generator->endAttribute( 'href' );
        $generator->endObjectElement( 'rootUserGroup' );

        // Media root location
        $generator->startObjectElement( 'rootMediaFolder', 'Location' );
        $generator->startAttribute( 'href', $this->router->generate( 'ezpublish_rest_loadLocation', array( 'locationPath' => '1/43' ) ) );
        $generator->endAttribute( 'href' );
        $generator->endObjectElement( 'rootMediaFolder' );

        // Location by remote ID
        // See comment above regarding usage of hash element
        $generator->startHashElement( 'locationByRemoteId' );
        $generator->startAttribute( 'media-type', '' );
        $generator->endAttribute( 'media-type' );
        $generator->startAttribute(
            'href',
            $this->templateRouter->generate(
                'ezpublish_rest_redirectLocation',
                array( 'remoteId' => '{remoteId}' )
            )
        );
        $generator->endAttribute( 'href' );
        $generator->endHashElement( 'locationByRemoteId' );

        $generator->startHashElement( 'locationByPath' );
        $generator->startAttribute( 'media-type', '' );
        $generator->endAttribute( 'media-type' );
        $generator->startAttribute(
            'href',
            $this->templateRouter->generate(
                'ezpublish_rest_redirectLocation',
                array( 'locationPath' => '{locationPath}' )
            )
        );
        $generator->endAttribute( 'href' );
        $generator->endHashElement( 'locationByPath' );

        // Trash
        $generator->startObjectElement( 'trash', 'Trash' );
        $generator->startAttribute( 'href', $this->router->generate( 'ezpublish_rest_loadTrashItems' ) );
        $generator->endAttribute( 'href' );
        $generator->endObjectElement( 'trash' );

        // Sections list
        $generator->startObjectElement( 'sections', 'SectionList' );
        $generator->startAttribute( 'href', $this->router->generate( 'ezpublish_rest_listSections' ) );
        $generator->endAttribute( 'href' );
        $generator->endObjectElement( 'sections' );

        // Content views
        $generator->startObjectElement( 'views', 'RefList' );
        $generator->startAttribute( 'href', $this->router->generate( 'ezpublish_rest_createView' ) );
        $generator->endAttribute( 'href' );
        $generator->endObjectElement( 'views' );

        // Object states groups
        $generator->startObjectElement( 'objectStateGroups', 'ObjectStateGroupList' );
        $generator->startAttribute( 'href', $this->router->generate( 'ezpublish_rest_loadObjectStateGroups' ) );
        $generator->endAttribute( 'href' );
        $generator->endObjectElement( 'objectStateGroups' );

        // Object states
        $generator->startObjectElement( 'objectStates', 'ObjectStateList' );
        $generator->startAttribute(
            'href',
            $this->templateRouter->generate(
                'ezpublish_rest_loadObjectStates',
                array( 'objectStateGroupId' => '{objectStateGroupId}' )
            )
        );
        $generator->endAttribute( 'href' );
        $generator->endObjectElement( 'objectStates' );

        // Global URL aliases
        $generator->startObjectElement( 'globalUrlAliases', 'UrlAliasRefList' );
        $generator->startAttribute( 'href', $this->router->generate( 'ezpublish_rest_listGlobalURLAliases' ) );
        $generator->endAttribute( 'href' );
        $generator->endObjectElement( 'globalUrlAliases' );

        // URL wildcards
        $generator->startObjectElement( 'urlWildcards', 'UrlWildcardList' );
        $generator->startAttribute( 'href', $this->router->generate( 'ezpublish_rest_listURLWildcards' ) );
        $generator->endAttribute( 'href' );
        $generator->endObjectElement( 'urlWildcards' );

        // Create session
        $generator->startObjectElement( 'createSession', 'UserSession' );
        $generator->startAttribute( 'href', $this->router->generate( 'ezpublish_rest_createSession' ) );
        $generator->endAttribute( 'href' );
        $generator->endObjectElement( 'createSession' );

        $generator->endObjectElement( 'Root' );
    }
}
