<?php
/**
 * File containing the URLWildcard ValueObjectVisitor class
 *
 * @copyright Copyright (C) 1999-2014 eZ Systems AS. All rights reserved.
 * @license http://ez.no/licenses/gnu_gpl GNU General Public License v2.0
 * @version 
 */

namespace eZ\Publish\Core\REST\Server\Output\ValueObjectVisitor;

use eZ\Publish\Core\REST\Common\Output\ValueObjectVisitor;
use eZ\Publish\Core\REST\Common\Output\Generator;
use eZ\Publish\Core\REST\Common\Output\Visitor;

/**
 * URLWildcard value object visitor
 */
class URLWildcard extends ValueObjectVisitor
{
    /**
     * Visit struct returned by controllers
     *
     * @param \eZ\Publish\Core\REST\Common\Output\Visitor $visitor
     * @param \eZ\Publish\Core\REST\Common\Output\Generator $generator
     * @param \eZ\Publish\API\Repository\Values\Content\URLWildcard $data
     */
    public function visit( Visitor $visitor, Generator $generator, $data )
    {
        $generator->startObjectElement( 'UrlWildcard' );
        $visitor->setHeader( 'Content-Type', $generator->getMediaType( 'UrlWildcard' ) );

        $generator->startAttribute(
            'href',
            $this->router->generate( 'ezpublish_rest_loadURLWildcard', array( 'urlWildcardId' => $data->id ) )
        );
        $generator->endAttribute( 'href' );

        $generator->startAttribute( 'id', $data->id );
        $generator->endAttribute( 'id' );

        $generator->startValueElement( 'sourceUrl', $data->sourceUrl );
        $generator->endValueElement( 'sourceUrl' );

        $generator->startValueElement( 'destinationUrl', $data->destinationUrl );
        $generator->endValueElement( 'destinationUrl' );

        $generator->startValueElement(
            'forward',
            $this->serializeBool( $generator, $data->forward )
        );
        $generator->endValueElement( 'forward' );

        $generator->endObjectElement( 'UrlWildcard' );
    }
}
