<?php
/**
 * File containing a test class
 *
 * @copyright Copyright (C) 1999-2014 eZ Systems AS. All rights reserved.
 * @license http://ez.no/licenses/gnu_gpl GNU General Public License v2.0
 * @version 
 */

namespace eZ\Publish\Core\REST\Server\Tests\Output\ValueObjectVisitor;

use eZ\Publish\Core\REST\Common\Tests\Output\ValueObjectVisitorBaseTest;

use eZ\Publish\Core\REST\Server\Output\ValueObjectVisitor;
use eZ\Publish\Core\REST\Server\Values\ContentList;
use eZ\Publish\Core\REST\Server\Values\RestContent;
use eZ\Publish\API\Repository\Values\Content\ContentInfo;
use eZ\Publish\Core\REST\Common;

class ContentListTest extends ValueObjectVisitorBaseTest
{
    /**
     * Test the ContentList visitor
     *
     * @return string
     */
    public function testVisit()
    {
        $visitor   = $this->getVisitor();
        $generator = $this->getGenerator();

        $generator->startDocument( null );

        $contentList = new ContentList( array() );

        $this->addRouteExpectation(
            'ezpublish_rest_redirectContent',
            array(),
            '/content/objects'
        );

        $visitor->visit(
            $this->getVisitorMock(),
            $generator,
            $contentList
        );

        $result = $generator->endDocument( null );

        $this->assertNotNull( $result );

        return $result;
    }

    /**
     * Test if result contains ContentList element
     *
     * @param string $result
     *
     * @depends testVisit
     */
    public function testResultContainsContentListElement( $result )
    {
        $this->assertTag(
            array(
                'tag' => 'ContentList',
            ),
            $result,
            'Invalid <ContentList> element.',
            false
        );
    }

    /**
     * Test if result contains ContentList element attributes
     *
     * @param string $result
     *
     * @depends testVisit
     */
    public function testResultContainsContentListAttributes( $result )
    {
        $this->assertTag(
            array(
                'tag'      => 'ContentList',
                'attributes' => array(
                    'media-type' => 'application/vnd.ez.api.ContentList+xml',
                    'href'       => '/content/objects',
                )
            ),
            $result,
            'Invalid <ContentList> attributes.',
            false
        );
    }

    /**
     * Test if ContentList visitor visits the children
     */
    public function testContentListVisitsChildren()
    {
        $visitor   = $this->getVisitor();
        $generator = $this->getGenerator();

        $generator->startDocument( null );

        $contentList = new ContentList(
            array(
                new RestContent( new ContentInfo() ),
                new RestContent( new ContentInfo() ),
            )
        );

        $this->getVisitorMock()->expects( $this->exactly( 2 ) )
            ->method( 'visitValueObject' )
            ->with( $this->isInstanceOf( 'eZ\\Publish\\Core\\Rest\\Server\\Values\\RestContent' ) );

        $visitor->visit(
            $this->getVisitorMock(),
            $generator,
            $contentList
        );
    }

    /**
     * Get the ContentList visitor
     *
     * @return \eZ\Publish\Core\REST\Server\Output\ValueObjectVisitor\ContentList
     */
    protected function internalGetVisitor()
    {
        return new ValueObjectVisitor\ContentList;
    }
}
