<?php
/**
 * File contains DateIntegrationTest class
 *
 * @copyright Copyright (C) 1999-2014 eZ Systems AS. All rights reserved.
 * @license http://ez.no/licenses/gnu_gpl GNU General Public License v2.0
 * @version 
 */

namespace eZ\Publish\SPI\Tests\FieldType;

use eZ\Publish\Core\Persistence\Legacy;
use eZ\Publish\Core\FieldType;
use eZ\Publish\SPI\Persistence\Content;

/**
 * Integration test for legacy storage field types
 *
 * This abstract base test case is supposed to be the base for field type
 * integration tests. It basically calls all involved methods in the field type
 * ``Converter`` and ``Storage`` implementations. Fo get it working implement
 * the abstract methods in a sensible way.
 *
 * The following actions are performed by this test using the custom field
 * type:
 *
 * - Create a new content type with the given field type
 * - Load create content type
 * - Create content object of new content type
 * - Load created content
 * - Copy created content
 * - Remove copied content
 *
 * @group integration
 */
class DateIntegrationTest extends BaseIntegrationTest
{
    /**
     * Get name of tested field type
     *
     * @return string
     */
    public function getTypeName()
    {
        return 'ezdate';
    }

    /**
     * Get handler with required custom field types registered
     *
     * @return \eZ\Publish\Core\Persistence\Legacy\Handler
     */
    public function getCustomHandler()
    {
        $handler = $this->getHandler();

        $fieldType = new FieldType\Date\Type();
        $fieldType->setTransformationProcessor( $this->getTransformationProcessor() );
        $handler->getFieldTypeRegistry()->register( 'ezdate', $fieldType );
        $handler->getStorageRegistry()->register(
            'ezdate',
            new FieldType\NullStorage()
        );
        $handler->getFieldValueConverterRegistry()->register(
            'ezdate',
            new Legacy\Content\FieldValue\Converter\Date()
        );

        return $handler;
    }

    /**
     * Returns the FieldTypeConstraints to be used to create a field definition
     * of the FieldType under test.
     *
     * @return \eZ\Publish\SPI\Persistence\Content\FieldTypeConstraints
     */
    public function getTypeConstraints()
    {
        return new Content\FieldTypeConstraints();
    }

    /**
     * Get field definition data values
     *
     * This is a PHPUnit data provider
     *
     * @return array
     */
    public function getFieldDefinitionData()
    {
        return array(
            // The ezdate field type does not have any special field definition
            // properties
            array( 'fieldType', 'ezdate' ),
            array(
                'fieldTypeConstraints',
                new Content\FieldTypeConstraints(
                    array(
                        'fieldSettings' => new FieldType\FieldSettings(
                            array(
                                'defaultType'  => 0,
                            )
                        ),
                    )
                )
            ),
        );
    }

    /**
     * Get initial field value
     *
     * @return \eZ\Publish\SPI\Persistence\Content\FieldValue
     */
    public function getInitialValue()
    {
        return new Content\FieldValue(
            array(
                'data'         => array(
                    'timestamp' => 123456,
                    'rfc850'    => null,
                ),
                'externalData' => null,
                'sortKey'      => 42,
            )
        );
    }

    /**
     * Get update field value.
     *
     * Use to update the field
     *
     * @return \eZ\Publish\SPI\Persistence\Content\FieldValue
     */
    public function getUpdatedValue()
    {
        return new Content\FieldValue(
            array(
                'data'         => array(
                    'timestamp' => 12345678,
                    'rfc850'    => null,
                ),
                'externalData' => null,
                'sortKey'      => 23,
            )
        );
    }
}
